@@ -62,6 +62,8 @@ typedef struct _ConfigFiles
     KConfig       *global;
     Bool          modified;
     unsigned int  watch;
+    unsigned int  kwinWatch;
+    unsigned int  globalWatch;
 }
 
 ConfigFiles;
@@ -165,14 +167,27 @@ const specialOptions[] =
     {"plane_to_11", "plane", "Switch to Desktop 11", true, OptionKey},
     {"plane_to_12", "plane", "Switch to Desktop 12", true, OptionKey},
 
-    {"next", "switcher", "Walk Through Windows", true, OptionKey},
-    {"prev", "switcher", "Walk Through Windows (Reverse)", true, OptionKey},
+    {"switch_to_1", "vpswitch", "Switch to Desktop 1", true, OptionKey},
+    {"switch_to_2", "vpswitch", "Switch to Desktop 2", true, OptionKey},
+    {"switch_to_3", "vpswitch", "Switch to Desktop 3", true, OptionKey},
+    {"switch_to_4", "vpswitch", "Switch to Desktop 4", true, OptionKey},
+    {"switch_to_5", "vpswitch", "Switch to Desktop 5", true, OptionKey},
+    {"switch_to_6", "vpswitch", "Switch to Desktop 6", true, OptionKey},
+    {"switch_to_7", "vpswitch", "Switch to Desktop 7", true, OptionKey},
+    {"switch_to_8", "vpswitch", "Switch to Desktop 8", true, OptionKey},
+    {"switch_to_9", "vpswitch", "Switch to Desktop 9", true, OptionKey},
+    {"switch_to_10", "vpswitch", "Switch to Desktop 10", true, OptionKey},
+    {"switch_to_11", "vpswitch", "Switch to Desktop 11", true, OptionKey},
+    {"switch_to_12", "vpswitch", "Switch to Desktop 12", true, OptionKey},
 
     {"autoraise", CORE_NAME, "AutoRaise", false, OptionBool},
     {"raise_on_click", CORE_NAME, "ClickRaise", false, OptionBool},
     {"snapoff_maximized", "move", "MoveResizeMaximizedWindows", false, OptionBool},
-
+    {"always_show", "resizeinfo", "GeometryTip", false, OptionBool},
+    {"allow_wraparound", "wall", "RollOverDesktops",  false, OptionBool},
+    
     {"autoraise_delay", CORE_NAME, "AutoRaiseInterval", false, OptionInt},
+    {"flip_time", "rotate", "ElectricBorderDelay", false, OptionInt},
 
     {"command_screenshot", CORE_NAME, NULL, true, OptionSpecial},
     {"command_window_screenshot", CORE_NAME, NULL, true, OptionSpecial},
@@ -182,11 +197,42 @@ const specialOptions[] =
     {"maximize_window_vertically", CORE_NAME, NULL, true, OptionSpecial},
     {"command11", CORE_NAME, NULL, true, OptionSpecial},
     {"click_to_focus", CORE_NAME, NULL, false, OptionSpecial},
+    {"mode", "resize", NULL, true, OptionSpecial},
     {"number_of_desktops", CORE_NAME, "Number", false, OptionSpecial},
+
+    {"snap_type", "snap", NULL, false, OptionSpecial},
+    {"edges_categories", "snap", NULL, false, OptionSpecial},
+    {"resistance_distance", "snap", NULL, false, OptionSpecial},
+    {"attraction_distance", "snap", NULL, false, OptionSpecial},
+
+    {"next", "switcher", "Walk Through Windows", true, OptionSpecial},
+    {"prev", "switcher", "Walk Through Windows (Reverse)", true, OptionSpecial},
+    {"next_all", "switcher", "Walk Through Windows", true, OptionSpecial},
+    {"prev_all", "switcher", "Walk Through Windows (Reverse)", true, OptionSpecial},
+    {"next_no_popup", "switcher", "Walk Through Windows", true, OptionSpecial},
+    {"prev_no_popup", "switcher", "Walk Through Windows (Reverse)", true, OptionSpecial},
+
+    {"edge_flip_pointer", "rotate", "ElectricBorders",  false, OptionSpecial},
+    {"edge_flip_window", "rotate", "ElectricBorders",  false, OptionSpecial},
+    {"edgeflip_pointer", "wall", "ElectricBorders",  false, OptionSpecial},
+    {"edgeflip_move", "wall", "ElectricBorders",  false, OptionSpecial},
+
+    {"mode", "place", "Placement",  false, OptionSpecial}
 };
 
 #define N_SOPTIONS (sizeof (specialOptions) / sizeof (struct _SpecialOption))
 
+static void
+createFile (QString name)
+{
+    if (!QFile::exists(name))
+    {
+	QFile file (name);
+	file.open (IO_WriteOnly | IO_Append);
+	file.close ();
+    }
+}
+
 static void
 reload (unsigned int,
 	void     *closure)
@@ -194,9 +240,15 @@ reload (unsigned int,
     CCSContext *context = (CCSContext *) closure;
     
     ccsDisableFileWatch (cFiles->watch);
+    ccsDisableFileWatch (cFiles->kwinWatch);
+    ccsDisableFileWatch (cFiles->globalWatch);
     cFiles->main->reparseConfiguration();
+    cFiles->kwin->reparseConfiguration();
+    cFiles->global->reparseConfiguration();
     ccsReadSettings (context);
     ccsEnableFileWatch (cFiles->watch);
+    ccsEnableFileWatch (cFiles->kwinWatch);
+    ccsEnableFileWatch (cFiles->globalWatch);
 }
 
 static bool
@@ -340,33 +392,183 @@ readIntegratedOption (CCSSetting *setting)
 	    KdeIntToCCS (setting, option);
 	    cFiles->kwin->setGroup ("Windows");
 	}
-	else if (specialOptions[option].settingName == "resize_mode")
+	else if (specialOptions[option].settingName == "mode" &&
+		 specialOptions[option].pluginName == "resize")
 	{
 	    QString mode = cFiles->kwin->readEntry ("ResizeMode");
-	    QString imode;
-	    QString result = "Normal";
+	    int     imode = -1;
+	    int     result = 0;
 
 	    if (cFiles->main->hasKey (specialOptions[option].settingName +
 		       		      " (Integrated)") )
-		imode = cFiles->main->readEntry (
+		imode = cFiles->main->readNumEntry (
 			specialOptions[option].settingName + " (Integrated)");
 
 	    if (mode == "Opaque")
 	    {
-		result = "Normal";
+		result = 0;
 
-		if (imode == "Stretch")
-		    result = "Stretch";
+		if (imode == 3)
+		    result = 3;
 	    }
 	    else if (mode == "Transparent")
 	    {
-		result = "Outline";
+		result = 1;
 
-		if (imode == "Filled Outline")
-		    result = "Filled Outline";
+		if (imode == 2)
+		    result = 2;
 	    }
 
-	    ccsSetString (setting, result.ascii() );
+	    ccsSetInt (setting, result);
+	}
+	else if (specialOptions[option].settingName == "snap_type")
+	{
+	    static int intList[2] = {0, 1};
+	    CCSSettingValueList list = ccsGetValueListFromIntArray (intList, 2,
+								    setting);
+	    ccsSetList (setting, list);
+	    ccsSettingValueListFree (list, TRUE);
+	}
+	else if (specialOptions[option].settingName == "resistance_distance" ||
+		 specialOptions[option].settingName == "attraction_distance")
+	{
+	    int val1 = cFiles->kwin->readNumEntry ("WindowSnapZone");
+	    int val2 = cFiles->kwin->readNumEntry ("BorderSnapZone");
+	    int result = KMAX (val1, val2);
+
+	    if (result == 0)
+		result =  cFiles->main->readNumEntry ("snap_distance (Integrated)");
+	    
+	    if (result > 0)
+	    	ccsSetInt (setting, result);
+	}
+	else if (specialOptions[option].settingName == "edges_categories")
+	{
+	    int val1 = cFiles->kwin->readNumEntry ("WindowSnapZone");
+	    int val2 = cFiles->kwin->readNumEntry ("BorderSnapZone");
+	    int intList[2] = {0, 0};
+	    int num = 0;
+
+	    if (val2 > 0)
+		num++;
+	    if (val1 > 0)
+	    {
+		intList[num] = 1;
+		num++;
+	    }
+
+	    CCSSettingValueList list = ccsGetValueListFromIntArray (intList,
+								    num,
+								    setting);
+	    ccsSetList (setting, list);
+	    ccsSettingValueListFree (list, TRUE);
+	}
+	else if (specialOptions[option].settingName == "next" ||
+		 specialOptions[option].settingName == "prev")
+	{
+	    bool val1;
+	    bool val2 = (cFiles->kwin->readEntry ("AltTabStyle") == "KDE");
+
+	    cFiles->kwin->setGroup ("TabBox");
+	    val1 = cFiles->kwin->readBoolEntry ("TraverseAll");
+	    cFiles->kwin->setGroup ("Windows");
+
+	    if (val2 && !val1)
+	    	KdeKeyToCCS (setting, option);
+	    else
+	    {
+		 CCSSettingActionValue action;
+
+		if (ccsGetAction (setting, &action) )
+		{
+		    action.keysym = 0;
+		    action.keyModMask = 0;
+		    ccsSetAction (setting, action);
+		}
+	    }
+	}
+	else if (specialOptions[option].settingName == "next_all" ||
+		 specialOptions[option].settingName == "prev_all")
+	{
+	    bool val1;
+	    bool val2 = (cFiles->kwin->readEntry ("AltTabStyle") == "KDE");
+
+	    cFiles->kwin->setGroup ("TabBox");
+	    val1 = cFiles->kwin->readBoolEntry ("TraverseAll");
+	    cFiles->kwin->setGroup ("Windows");
+
+	    if (val2 && val1)
+	    	KdeKeyToCCS (setting, option);
+	    else
+	    {
+		 CCSSettingActionValue action;
+
+		if (ccsGetAction (setting, &action) )
+		{
+		    action.keysym = 0;
+		    action.keyModMask = 0;
+		    ccsSetAction (setting, action);
+		}
+	    }
+	}
+	else if (specialOptions[option].settingName == "next_no_popup" ||
+		 specialOptions[option].settingName == "prev_no_popup")
+	{
+	    bool val2 = (cFiles->kwin->readEntry ("AltTabStyle") == "KDE");
+
+	    if (!val2)
+	    	KdeKeyToCCS (setting, option);
+	    else
+	    {
+		 CCSSettingActionValue action;
+
+		if (ccsGetAction (setting, &action) )
+		{
+		    action.keysym = 0;
+		    action.keyModMask = 0;
+		    ccsSetAction (setting, action);
+		}
+	    }
+	}
+	else if (specialOptions[option].settingName == "edge_flip_window" ||
+		 specialOptions[option].settingName == "edgeflip_move")
+	{
+	    int val = cFiles->kwin->readNumEntry ("ElectricBorders");
+
+	    if (val > 0)
+		ccsSetBool (setting, TRUE);
+	    else
+		ccsSetBool (setting, FALSE);
+	}
+	else if (specialOptions[option].settingName == "edge_flip_pointer" ||
+		 specialOptions[option].settingName == "edgeflip_pointer")
+	{
+	    int val = cFiles->kwin->readNumEntry ("ElectricBorders");
+
+	    if (val > 1)
+		ccsSetBool (setting, TRUE);
+	    else
+		ccsSetBool (setting, FALSE);
+	}
+	else if (specialOptions[option].settingName == "mode" &&
+		 specialOptions[option].pluginName == "place")
+	{
+	    QString mode = cFiles->kwin->readEntry ("Placement");
+	    int     result = 0;
+
+
+	    if (mode == "Smart")
+		result = 2;
+	    else if (mode == "Maximizing")
+		result = 3;
+	    else if (mode == "Cascade")
+		result = 0;
+	    else if (mode == "Random")
+		result = 4;
+	    else if (mode == "Centered")
+		result = 1;
+
+	    ccsSetInt (setting, result);
 	}
 	break;
 
@@ -430,6 +632,11 @@ getSettingIsReadOnly (CCSSetting *setting)
 	{
 	    return TRUE;
 	}
+	else if (specialOptions[option].settingName == "snap_type" ||
+		 specialOptions[option].settingName == "attraction_distance")
+	{
+	    return TRUE;
+	}
 	break;
 
     default:
@@ -457,9 +664,9 @@ getExistingProfiles (CCSContext *)
     {
 	QString str = (*it);
 
-	if (str.length() > 8)
+	if (str.length() > 9)
 	{
-	    QString profile = str.right (str.length() - 8);
+	    QString profile = str.right (str.length() - 9);
 
 	    if (!profile.isEmpty() )
 		ret = ccsStringListAppend (ret, strdup (profile.ascii() ) );
@@ -972,6 +1179,213 @@ writeIntegratedOption (CCSSetting *setting)
 	    CCSIntToKde (setting, option);
 	    cFiles->kwin->setGroup ("Windows");
 	}
+	if (specialOptions[option].settingName == "mode" &&
+	    specialOptions[option].pluginName == "resize")
+	{
+		QString mode = cFiles->kwin->readEntry("ResizeMode");
+		QString val = "Opaque";
+		int     iVal;
+		if (ccsGetInt(setting, &iVal) && (iVal == 1 || iVal == 2))
+		{
+			val = "Transparent";
+		}
+		if (mode != val)
+		{
+			cFiles->modified = true;
+			cFiles->kwin->writeEntry("ResizeMode",val);
+		}
+		cFiles->main->writeEntry(specialOptions[option].settingName + " (Integrated)",iVal);
+	}
+
+	if (specialOptions[option].settingName == "resistance_distance" ||
+	    specialOptions[option].settingName == "edges_categories")
+	{
+	    int *values, numValues;
+	    CCSSettingValueList sList;
+
+	    bool edge = false;
+	    bool window = false;
+
+	    int iVal = 0;
+
+	    CCSSetting *edges = ccsFindSetting(setting->parent,
+					       "edges_categories",
+					       setting->isScreen,
+					       setting->screenNum);
+
+	    CCSSetting *dist = ccsFindSetting(setting->parent,
+					      "resistance_distance",
+					      setting->isScreen,
+					      setting->screenNum);
+
+	    if (!edges || !dist || !ccsGetList (edges, &sList) ||
+		!ccsGetInt(dist, &iVal))
+		break;
+
+	    values = ccsGetIntArrayFromValueList (sList, &numValues);
+
+	    for (int i = 0; i < numValues; i++)
+	    {
+		if (values[i] == 0)
+		    edge = true;
+		if (values[i] == 1)
+		    window = true;
+	    }
+
+	    if (values)
+		free (values);
+
+	    if (edge)
+		cFiles->kwin->writeEntry ("BorderSnapZone", iVal);
+	    else
+		cFiles->kwin->writeEntry ("BorderSnapZone", 0);
+	    
+	    if (window)
+		cFiles->kwin->writeEntry ("WindowSnapZone", iVal);
+	    else
+		cFiles->kwin->writeEntry ("WindowSnapZone", 0);
+	    if (window | edge)
+		cFiles->modified = true;
+	    cFiles->main->writeEntry ("snap_distance (Integrated)",iVal);
+	}
+	else if (specialOptions[option].settingName == "next" ||
+		 specialOptions[option].settingName == "prev")
+	{
+	    CCSSettingActionValue action;
+
+	    if (!ccsGetAction (setting, &action))
+		break;
+
+	    if (action.keysym == 0 && action.keyModMask == 0)
+		break;
+
+	    CCSKeyToKde (setting, option);
+
+	    cFiles->kwin->setGroup ("TabBox");
+	    cFiles->kwin->writeEntry ("TraverseAll", false);
+	    cFiles->kwin->setGroup ("Windows");
+	    
+	    cFiles->kwin->writeEntry ("AltTabStyle", "KDE");
+
+	    cFiles->modified = true;
+	}
+	else if (specialOptions[option].settingName == "next_all" ||
+		 specialOptions[option].settingName == "prev_all")
+	{
+	    CCSSettingActionValue action;
+
+	    if (!ccsGetAction (setting, &action))
+		break;
+
+	    if (action.keysym == 0 && action.keyModMask == 0)
+		break;
+
+	    CCSKeyToKde (setting, option);
+
+	    cFiles->kwin->setGroup ("TabBox");
+	    cFiles->kwin->writeEntry ("TraverseAll", true);
+	    cFiles->kwin->setGroup ("Windows");
+	    
+	    cFiles->kwin->writeEntry ("AltTabStyle", "KDE");
+
+	    cFiles->modified = true;
+	}
+	else if (specialOptions[option].settingName == "next_no_popup" ||
+		 specialOptions[option].settingName == "prev_no_popup")
+	{
+	    CCSSettingActionValue action;
+
+	    if (!ccsGetAction (setting, &action))
+		break;
+
+	    if (action.keysym == 0 && action.keyModMask == 0)
+		break;
+
+	    CCSKeyToKde (setting, option);
+
+	    cFiles->kwin->writeEntry ("AltTabStyle", "CDE");
+
+	    cFiles->modified = true;
+	}
+	else if (specialOptions[option].settingName == "edge_flip_window" ||
+		 specialOptions[option].settingName == "edgeflip_move")
+	{
+	    int  oVal = cFiles->kwin->readNumEntry ("ElectricBorders");
+	    Bool val;
+
+	    if (!ccsGetBool (setting, &val))
+		break;
+
+	    if (val)
+		cFiles->kwin->writeEntry ("ElectricBorders", KMAX (1, oVal));
+	    else
+		cFiles->kwin->writeEntry ("ElectricBorders", 0);
+	    cFiles->modified = true;
+	}
+	else if (specialOptions[option].settingName == "edge_flip_pointer" ||
+		 specialOptions[option].settingName == "edgeflip_pointer")
+	{
+	    int  oVal = 0;
+	    Bool val, val2;
+
+	    if (!ccsGetBool (setting, &val))
+		break;
+
+
+	    CCSSetting *valSet = ccsFindSetting(setting->parent,
+						 "edge_flip_window",
+						 setting->isScreen,
+						 setting->screenNum);
+
+	    if (!valSet)
+	     	valSet = ccsFindSetting(setting->parent, "edgeflip_move",
+					setting->isScreen, setting->screenNum);
+
+	    if (valSet && ccsGetBool (valSet, &val2))
+	    {
+		if (val2)
+		    oVal = 1;
+	    }
+	    else
+		oVal = 0;
+		
+
+	    if (val)
+		cFiles->kwin->writeEntry ("ElectricBorders", 2);
+	    else
+		cFiles->kwin->writeEntry ("ElectricBorders", oVal);
+	    cFiles->modified = true;
+	}
+	else if (specialOptions[option].settingName == "mode" &&
+		 specialOptions[option].pluginName == "place")
+	{
+	    int val;
+	    if (!ccsGetInt (setting, &val))
+		break;
+
+	    switch (val)
+	    {
+	    case 0:
+		cFiles->kwin->writeEntry ("Placement", "Cascade");
+		break;
+	    case 1:
+		cFiles->kwin->writeEntry ("Placement", "Centered");
+		break;
+	    case 2:
+		cFiles->kwin->writeEntry ("Placement", "Smart");
+		break;
+	    case 3:
+		cFiles->kwin->writeEntry ("Placement", "Maximizing");
+		break;
+	    case 4:
+		cFiles->kwin->writeEntry ("Placement", "Random");
+		break;
+	    default:
+		break;
+	    }
+
+	    cFiles->modified = true;
+	}
 	break;
 
     default:
@@ -1003,7 +1417,7 @@ writeSetting (CCSContext *c,
 	return;
     }
 
-    if (setting->isDefault && key != "____plugin_enabled")
+    if (setting->isDefault)
     {
 	if (cfg->hasKey (key) )
 	    cfg->deleteEntry (key);
@@ -1325,10 +1739,12 @@ readInit (CCSContext *c)
 
 	delete cFiles->main;
 
-	cFiles->main = new KSimpleConfig (configName);
-	ccsRemoveFileWatch (cFiles->watch);
 	QString wFile = KGlobal::dirs()->saveLocation ("config",
 			QString::null, false) + configName;
+	createFile (wFile);
+	
+	cFiles->main = new KSimpleConfig (configName);
+	ccsRemoveFileWatch (cFiles->watch);
 	cFiles->watch = ccsAddFileWatch (wFile.ascii(), TRUE,
 					 reload, (void *) c);
     }
@@ -1344,7 +1760,7 @@ static Bool
 writeInit (CCSContext *c)
 {
     if (!instance)
-	instance = new KInstance ("beryl-kconfig");
+	instance = new KInstance ("ccs-backend-kconfig");
 
     if (cFiles->profile != ccsGetProfile (c) )
     {
@@ -1359,15 +1775,20 @@ writeInit (CCSContext *c)
 
 	delete cFiles->main;
 
-	cFiles->main = new KSimpleConfig (configName);
-	ccsRemoveFileWatch (cFiles->watch);
 	QString wFile = KGlobal::dirs()->saveLocation ("config",
 			QString::null, false) + configName;
+	
+	createFile (wFile);
+	
+	cFiles->main = new KSimpleConfig (configName);
+	ccsRemoveFileWatch (cFiles->watch);
 	cFiles->watch = ccsAddFileWatch (wFile.ascii(), TRUE,
 					 reload, (void *) c);
     }
 
     ccsDisableFileWatch (cFiles->watch);
+    ccsDisableFileWatch (cFiles->kwinWatch);
+    ccsDisableFileWatch (cFiles->globalWatch);
 
     return TRUE;
 }
@@ -1376,7 +1797,19 @@ static void
 writeDone (CCSContext *)
 {
     cFiles->main->sync();
+    if (cFiles->modified)
+    {
+	cFiles->kwin->sync();
+	cFiles->global->sync();
+	DCOPClient *client = kapp->dcopClient();
+	if (!client->isAttached())
+	client->attach();
+	client->send("kwin", "KWinInterface", "reconfigure()", "");
+	cFiles->modified = false;
+    }
     ccsEnableFileWatch (cFiles->watch);
+    ccsEnableFileWatch (cFiles->kwinWatch);
+    ccsEnableFileWatch (cFiles->globalWatch);
 }
 
 
@@ -1397,17 +1830,29 @@ init (CCSContext *c)
 	cFiles->profile = ccsGetProfile (c);
     }
 
-    cFiles->main = new KSimpleConfig (configName);
-    cFiles->kwin   = new KConfig ("kwinrc", true);
-    cFiles->global = new KConfig ("kdeglobals", true);
-    
-    cFiles->kwin->setGroup ("Windows");
-    cFiles->global->setGroup ("Global Shortcuts");
-    
     QString wFile = KGlobal::dirs()->saveLocation ("config",
 		    QString::null, false) + configName;
+
+    createFile (wFile);
+
+    cFiles->main = new KSimpleConfig (configName);
+    cFiles->kwin   = new KConfig ("kwinrc");
+    cFiles->global = new KConfig ("kdeglobals");
+
+    cFiles->kwin->setGroup ("Windows");
+    cFiles->global->setGroup ("Global Shortcuts");
+
     cFiles->watch = ccsAddFileWatch (wFile.ascii(), TRUE, reload, (void *) c);
 
+    wFile = KGlobal::dirs()->saveLocation ("config",
+		    QString::null, false) + "kwinrc";
+    cFiles->kwinWatch = ccsAddFileWatch (wFile.ascii(), TRUE, reload,
+					 (void *) c);
+    wFile = KGlobal::dirs()->saveLocation ("config",
+		    QString::null, false) + "kdeglobals";
+    cFiles->globalWatch = ccsAddFileWatch (wFile.ascii(), TRUE, reload,
+					   (void *) c);
+
     return TRUE;
 }
 
@@ -1416,6 +1861,10 @@ fini (CCSContext *)
 {
     if (cFiles)
     {
+	ccsRemoveFileWatch (cFiles->watch);
+	ccsRemoveFileWatch (cFiles->kwinWatch);
+	ccsRemoveFileWatch (cFiles->globalWatch);
+
 	if (cFiles->main)
 	    delete cFiles->main;
 
@@ -1425,8 +1874,6 @@ fini (CCSContext *)
 	if (cFiles->global)
 	    delete cFiles->global;
 
-	ccsRemoveFileWatch (cFiles->watch);
-
 	delete cFiles;
     }
 
